#
# FNET/ICB CLIENT 1.1   11/18/96
# John M Vinopal        banshee@resort.com
#
# Copyright (C) 1996, The Resort, All Rights Reserved.
# Permission is granted to copy and modify this program for
# non-commercial purposes, so long as this copyright notice is
# preserved.  This software is distributed without warranty.
#

package FNET;
require 5.000;
use Socket;
use FileHandle;
# XXX - use only for testing.
#use strict;
#use diagnostics;

my $A = "\001";	# Packet argument delimiter.

$FNET'M_LOGIN		= 'a';     # login packet */
$FNET'M_LOGINOK		= 'a';     # login packet */
$FNET'M_OPEN		= 'b';     # open msg to group */
$FNET'M_PERSONAL	= 'c';     # personal msg */
$FNET'M_STATUS		= 'd';     # status update message */
$FNET'M_ERROR		= 'e';     # error message */
$FNET'M_IMPORTANT	= 'f';     # special important announcement */
$FNET'M_EXIT		= 'g';     # tell other side to exit */
$FNET'M_COMMAND		= 'h';     # send a command from user */
$FNET'M_CMDOUT		= 'i';     # output from a command */
$FNET'M_PROTO		= 'j';     # protocol version information */
$FNET'M_BEEP		= 'k';     # beeps */
$FNET'M_PING		= 'l';     # ping packet */
$FNET'M_PONG		= 'm';     # return for ping packet */
$FNET'M_OOPEN		= 'n';     # for own open messages */
$FNET'M_OPERSONAL	= 'o';     # for own personal messages */


#########################################################
#
#	tcpopen(SOCKET,hostname,portnumber);
#
#	locate the machine named "hostname" on the internet,
#	and connect a socket to that machine on the tcp port
#	specified, by number.
#
#	return "ok" of succeeded or nothing if fail.

sub	tcpopen
{
	my ($hostname,$portnumber) = @_;

	my ($SOCK, $iaddr,$paddr,$proto,$result);

	$SOCK = new FileHandle;
	$iaddr = inet_aton($hostname) || return;
	$paddr = sockaddr_in($portnumber,$iaddr) || return;
	$proto = getprotobyname('tcp') || return;
	socket($SOCK, PF_INET, SOCK_STREAM, $proto) || return;
	connect($SOCK, $paddr) || return;
	my $sel = select($SOCK); $| = 1; $SOCK = select($sel);

	return $SOCK;
}

#
# Open the tcp connection and send our login packet.
#
sub	login
{
	my ($hostname,$portnumber,$id,$nick,$group,$cmd,$passwd) = @_;
	my ($SOCKET);

	if (not ($SOCKET = tcpopen($hostname,$portnumber))) {
		print "Failed to connnect.\n";
		return;
	}
	if (not (sendlogin($SOCKET, $id, $nick, $group, $cmd, $passwd))) {
		print "Login failed\n";
		return;
	}
	return $SOCKET;
}

#sub	sendlogin(id, nick, group, command, passwd)
# Sends a login packet to the server.  It specifies our login name,
# nickname, active group, a command "login" or "w", and our passwd.
sub	sendlogin
{
	my	($SOCK, $id, $nick, $group, $cmd, $passwd) = @_;
	my	($pbuf);
	
	$pbuf = $FNET'M_LOGIN.$id.$A.$nick.$A.$group.$A.$cmd.$A.$passwd;
	if (sendit($SOCK, $pbuf)) {
		return "ok";
	}
}

sub	csendopen
{
	my	($SOCK, $txt) = @_;
	my	($pbuf);
	
	$pbuf = $FNET'M_OPEN.$txt;
	if (sendit($SOCK, $pbuf)) {
		return "ok";
	}
}

sub	sendpong
{
	my	($SOCK) = @_;
	my	($pbuf) = $FNET'M_PONG;

	if (sendit($SOCK, $pbuf)) {
		return "ok";
	}
}


#sub	sendcmd(cmd, args)
sub	sendcmd
{
	my	($SOCK, $cmd, $args) = @_;
	my	($pbuf);

	if (defined($args)) {
		$pbuf = $FNET'M_COMMAND.$cmd.$A.$args;
	} else {
		$pbuf = $FNET'M_COMMAND.$cmd.$A;
	}

	if (sendit($SOCK, $pbuf)) {
		return "ok";
	}
}


sub	sendit
{
	my	($SOCK, $packet) = @_;
	my	($plen) = length($packet) + 1;
	my	($wrotelen);

	# Add the packet length to the packet head.
	$packet = chr($plen).$packet."\0";
	$plen = length($packet);

	$wrotelen = send($SOCK, $packet, 0);
	if (not defined($wrotelen)) {
		print "error on send\n";
	} elsif ($wrotelen != $plen) {
		print "error: wrote $wrotelen of $plen\n";
	} else {
		return "ok";
	}
	return;
}


#
#
sub	readit
{
	my ($SOCK) = @_;
	my ($sbuf, $buffer, $pbuf, $ret);
	$sbuf = "";
	$buffer = "";
	$pbuf = "";
	$ret = 0;

	$ret = recv($SOCK, $sbuf, 1, 0);
	if (not defined($ret)) {
		print "error on size read\n";
	} elsif (length($sbuf) != 1) {
		print "error: size read ".length($pbuf)." of 1\n";
	} else {
		# Convert char to integer.
		$sbuf = ord($sbuf);
		while ($sbuf) {
			$ret = recv($SOCK, $pbuf, $sbuf, 0);
			if (not defined($ret)) {
				print "error on msg read\n";
			} else {
				$sbuf -= length($pbuf);
				$buffer = $buffer.$pbuf;
			}
		}
		return $buffer;
	}
	return;
}

sub	splitpacket
{
	my	($message) = @_;

	my @msgarray = split($A, $message);
	return(@msgarray);
}


sub	formatwholine
{
	my	(@args) = @_;
	my	($idle, $relay);

	# Moderator
	if ($args[0] eq "m") {
		$args[0] = "*";
	} else {
		$args[0] = " ";
	}

	# Idle time in seconds.
	if ($args[2] < 60) {
		$idle = "-";
	} else {
		$args[2] /= 60;
		$idle = sprintf("%.0dm", $args[2]);
	}

	# Relay time?
	if ($args[3] < 60) {
		$relay = "-";
	} else {
		$args[3] /= 60;
		$relay = sprintf("%.0dm", $args[3]);
	}

	# Translate time into english string.
	my @time = localtime($args[4]);
	$args[4] = "am";
	if ($time[1] < 10) {
		$time[1] = "0".$time[1];
	}
	if ($time[2] > 12) {
		$time[2] -= 12;
		$args[4] = "pm";
	} elsif ($time[2] < 1) {
		$time[2] = 12;
	}
		
	my $str = sprintf("  %s%-12s  %3s   %3s  %2s:%2s%s  %s@%s  %s",
		$args[0], $args[1],
		$idle, $relay,
		$time[2], $time[1], $args[4],
		$args[5], $args[6], $args[7]);

	return $str;
}

1;
