#!/usr/local/bin/perl
#
# FNET/ICB CLIENT 1.1	11/18/96
# John M Vinopal        banshee@resort.com
#
# Copyright (C) 1996, The Resort, All Rights Reserved.
# Permission is granted to copy and modify this program for
# non-commercial purposes, so long as this copyright notice is
# preserved.  This software is distributed without warranty.

use FNET;
use CMDLINE;
require "getopts.pl";
require 5.000;
# XXX - use only for testing.
#use strict;
#use diagnostics;

my $version = "Perl Fnet Client 1.1";
my $LINESIZE = 78;
my $SCREENSIZE = 24;

# Machine Specifics.
my	($os, $EOL);
eval '$os = `uname`';
$os = 'Unix' if $os;  # if you just need generic OS
$os = 'Mac' if defined( $MacPerl'Version );
$os = 'Dos' unless $os;

if ($os eq "Mac") {
	$EOL = "\n";
} else {
	# XXX - no idea if this is ok for DOS.
	$EOL = "\r\n";
}

# Default Server Variables.
my	($hostname, $portnumber, $Sock);
$hostname = "klinzhai.evolve.com";
$portnumber = 7326;
$Sock = -1;

# Default Session Variables.
# MacPerl doesn't have getlogin().
my	($name, $nick, $group, $cmd, $pass);
$name = "";
if (not $name = eval("getlogin()")) {
	$name = "user";
}
$nick = $name;
$group = "1";
$cmd = "login";
$pass = "";

my	(@privlist, @openlist, $cmdchar, $pagemode, $away, $alertmode);
$cmdchar = "/";
$pagemode = 0;
$away = "";
$alertmode = 0;

# XXX - for 'strict'
# my ($opt_g, $opt_N, $opt_n, $opt_s, $opt_w, $opt_h);

# Process command line options.
Getopts("hwg:N:n:s:");
if ($opt_g) { $group = $opt_g; }
if ($opt_N) { $name = $opt_N; $nick = $name; }
if ($opt_n) { $nick = $opt_n; }
if ($opt_s) { $hostname = $opt_s; }
if ($opt_w) { $opt_w = ""; $cmd = "w"; }
if ($opt_h) {
	$opt_h = "";
	print "Usage: $0 [-n nickname] [-g groupname] [-s server] [-w]\n";
	exit;
}

print "Awaiting connection to '$hostname'\n";

# Connect to the server.
if (not $Sock = 
	FNET::login($hostname,$portnumber, $name, $nick, $group, $cmd, $pass)) {
	quit();
}

# Set up select to watch the fds.
my ($rin, $win, $ein, $rout, $wout, $eout);
$rin = $win = $ein = '';
vec($rin,fileno(STDIN),1) = 1;
vec($rin,fileno($Sock),1) = 1;
$ein = $rin | $win;

$SIG{'INT'} = 'doKill';
# Loop forever, processing STDIN and the Socket.
while (1) {
LOOP:
	# Set us to catch the users first character.
	system('stty raw -echo');
	$SIG{'TSTP'} = 'doSuspend';

	select($rout=$rin, $wout=$win, $eout=$ein, undef);

	# Process user input.
	if (vec($rout, fileno(STDIN), 1) == 1) {
		my	($msg);

		$msg = CMDLINE::cmdline($EOL, $os);
		# Ignore null messages.
		if ($msg eq "") {
			goto LOOP;
		}
		print "$EOL";
		if (substr($msg,0,1) ne $cmdchar) {
			# Send a normal message.
			push(@openlist, "--> $msg");
			FNET::csendopen($Sock, $msg);
		} else {
			processCmd($msg);
		}
	}

	# Process incoming data on the socket.
	if (vec($rout, fileno($Sock), 1) == 1) {
		processsock($Sock);
	}
}
quit();
exit;


#
# User sending a command.
#
sub processCmd
{
	my	($msg) = @_;
	my	($cmdpos, $cmd, $arg);

	$cmdpos = index($msg," ");
	if ($cmdpos > 1) {
		$cmd = substr($msg, 1, $cmdpos-1);
		$arg = substr($msg, $cmdpos+1);
	} else {
		$cmd = substr($msg, 1);
	}

	# Process some command locally, send the others.
	if ($cmd eq "q" or $cmd eq "quit") {
		quit();
	} elsif ($cmd eq "t" or $cmd eq "time") {
		print localtime()."$EOL";

	} elsif ($cmd eq "alert") {
		if ($alertmode eq 0) {
			$alertmode = 1;
			print "[=CStatus=] Alert turned on.$EOL";
		} else {
			print "[=CError=] Alert already on.$EOL";
		}

	} elsif ($cmd eq "noalert") {
		if ($alertmode ne 0) {
			$alertmode = 0;
			print "[=CStatus=] Alert turned off.$EOL";
		} else {
			print "[=CError=] Alert already off.$EOL";
		}

	} elsif ($cmd eq "away") {
		if (not $arg) {
			if (not $away) {
				print "[=CStatus=] Away was not set.$EOL";
			} else {
				$away = "";
				print "[=CStatus=] Away unset.$EOL";
			}
		} else {
			$away = "[=Away=] ".$arg;
			print "[=CStatus=] Away message set to '$away'.$EOL";
		}

	} elsif ($cmd eq "?" or $cmd eq "help") {
		print "$version Command Help:$EOL";
		print "----------Command-------------------";
		print "----------Examples------------------$EOL";
		print "type to send a message to your group.";
		print "\thi everyone!$EOL";
		print "/w . to see whos in your group.$EOL";
		print "also: /w <.|group|\@user>"; 
		print "\t\t/w \@banshee  or /w 1$EOL"; 
		print "/g changes your group."; 
		print "\t\t\t/g group2 or /g \@umlaut$EOL"; 
		print "/m sends a private message.";
		print "\t\t/m izzy hi mom!  I'm in jail!$EOL";
		print "/beep sends a beep.";
		print "\t\t\t/beep sl'lee$EOL";
		print "/nobeep prevents incoming beeps.";
		print "\t/nobeep on or /nobeep off$EOL";
		print "/hush ignores a user.";
		print "\t\t\t/hush andrew17$EOL";
		print "Try also /time, /m server help, ";
		print "/display, /personal, /command$EOL";
		print "------------------------------------";
		print "------------------------------------$EOL";
	} elsif ($cmd eq "command") {
		if (not $arg) {
			print "Command prefix is '$cmdchar'.$EOL";
		} else {
			print "Command prefix changed from '$cmdchar' to ";
			$cmdchar = substr($arg, 0, 1);
			print "'$cmdchar'.$EOL";
		}

	} elsif ($cmd eq "page") {
		if ($pagemode eq -1) {
			$pagemode = 0;
			print "[=CStatus=] Hold-page turned on.$EOL";
		} else {
			print "[=CError=] Hold-page already on.$EOL";
		}

	} elsif ($cmd eq "nopage") {
		if ($pagemode ne -1) {
			$pagemode = -1;
			print "[=CStatus=] Hold-page turned off.$EOL";
		} else {
			print "[=CError=] Hold-page already off.$EOL";
		}

	} elsif ($cmd eq "personal") {
		my ($i) = 0;
		if (not $arg) { $arg = 10; }
		if ($arg > $#privlist) { $i = 0; }
		else { $i = $#privlist - $arg + 1; }
		print "--START--$EOL";
		while ($i <= $#privlist) {
			print "$privlist[$i]$EOL";
			$i++;
		}
		print "--END--$EOL";
		
	} elsif ($cmd eq "display") {
		my ($i) = 0;
		if (not $arg) { $arg = 10; }
		if ($arg > $#openlist) { $i = 0; }
		else { $i = $#openlist - $arg + 1; }
		print "--START--$EOL";
		while ($i <= $#openlist) {
			print "$openlist[$i]$EOL";
			$i++;
		}
		print "--END--$EOL";
		
	} elsif ($cmd eq "flirt") {
		# XXX - need to read flirt file.
		FNET::sendcmd($Sock, "m", $arg." [=Flirt!=]");

	} elsif ($cmd eq "btth") {
		FNET::sendcmd($Sock, "m", $arg." [=Whack!=] boot to the head!");
	} elsif ($cmd eq "hug") {
		FNET::sendcmd($Sock, "m", $arg." [=HUG!=] you've been hugged! :)");

	} else {
		if ($cmd eq "who") {
			$cmd = "w";
		}

		# Private Message Review.
		if ($cmd eq "m") {
			my ($name, $msg) = split(' ', $arg, 2);
			push(@privlist, "-{to $name}-> ".$msg);
			push(@openlist, "-{to $name}-> ".$msg);
		}

		FNET::sendcmd($Sock, $cmd, $arg);
	}
}

#
# The core of interactive fnet.  Takes a server packet
# and formats it into the common fnet output.
sub	processsock
{
	my	($SOCK) = @_;
	my	($msgpack, $packettype, @msgarray);

	$msgpack = FNET::readit($SOCK);
	if (not $msgpack) {
		print "Failed on socket read.$EOL";
		quit();
	}
	$packettype = substr($msgpack, 0, 1);
	@msgarray = FNET::splitpacket(substr($msgpack,1));

	if ($packettype eq $FNET'M_LOGINOK) {
		print "[=Ack=] login ok!$EOL";
	} elsif ($packettype eq $FNET'M_OPEN) {
		if ($alertmode == 1) {
			print "\007";
		}
		push(@openlist, "<$msgarray[0]> ".$msgarray[1]);
		formatmessage("<$msgarray[0]>", $msgarray[1]);
	} elsif ($packettype eq $FNET'M_PERSONAL) {
		if ($alertmode == 1) {
			print "\007";
		}
		if ($away) {
			FNET::sendcmd($Sock, "m", $msgarray[0]." ".$away);
		}
		push(@privlist, "<*$msgarray[0]*> ".$msgarray[1]);
		push(@openlist, "<*$msgarray[0]*> ".$msgarray[1]);
		formatmessage("<*$msgarray[0]*>", $msgarray[1]);
	} elsif ($packettype eq $FNET'M_STATUS) {
		print "[=$msgarray[0]=] $msgarray[1]$EOL";
	} elsif ($packettype eq $FNET'M_ERROR) {
		print "[=Error=] $msgarray[0]$EOL";
	} elsif ($packettype eq $FNET'M_IMPORTANT) {
		print "[=Hey!=]\007 $msgarray[1]$EOL";
	} elsif ($packettype eq $FNET'M_EXIT) {
		print "Server sent exit.$EOL";
		quit();
	} elsif ($packettype eq $FNET'M_CMDOUT) {
		formatcommandout(@msgarray);
	} elsif ($packettype eq $FNET'M_PROTO) {
		print "Connected to the $msgarray[1] ICB server ($msgarray[2])$EOL";
	} elsif ($packettype eq $FNET'M_BEEP) {
		print "[=Beep=]\007 $msgarray[0] sent you a beep!$EOL";
	} elsif ($packettype eq $FNET'M_PING) {
		print "WARN: ping packet$EOL";
	} elsif ($packettype eq $FNET'M_PONG) {
		print "WARN: pong$EOL";
	} elsif ($packettype eq $FNET'M_OOPEN) {
		print "WARN: own open$EOL";
	} elsif ($packettype eq $FNET'M_OPERSONAL) {
		print "WARN: own personal$EOL";
	} else {
		print "WARN: bad packet type $packettype$EOL";
	}

	# XXX - for page mode.
	if ($pagemode ne -1) {
		if ($pagemode == $SCREENSIZE) {
			print "--More--";
			# XXX - macs return nulls on read.
			while (getc() eq "") {;}
			print "\010"x8;
			print "\040"x8;
			print "\010"x8;
			$pagemode = 0;
		}
		$pagemode++;
	}
}

#
# Format open and private messages, breaking to fit on the screen.
#
sub	formatmessage
{
	my	($from, $message) = @_;
	my	$msgsize = length($message);
	my	$linesize = $LINESIZE - length($from);

	# break every 80 chars.
	if ($msgsize > $linesize) {
		my	$start = 0;
		my	$end = 0;
		# Stick a space at the end so we'll grab it.
		$message = $message." ";
		while ($start < $msgsize) {
			# Look for a space somewhere near the EOL break.
			$end = rindex($message, " ", $start + $linesize);
			# If no space, we've got to break it manually.
			if ($end < $start) {
				$end = $start + $linesize - 1;
				# Don't try to take too much.
				if ($end > $msgsize) {
					$end = $msgsize;
				}
			}
			print "$from ", substr($message, $start, ($end-$start)), "$EOL";
			$start = $end + 1;
		}
	} else {
		print "$from $message$EOL";
	}
}

#
# Format the command output from the server.
#
sub	formatcommandout
{
	my	($cmd, @args) = @_;

	if ($cmd eq "co") {
		print @args;
	} elsif ($cmd eq "wh\000") {
		print "   Nickname     Idle  Resp  Sign-On  Account";
	} elsif ($cmd eq "wl") {
		print FNET::formatwholine(@args);
	} else {
		print "WARN: command -$cmd-: @args";
	}
	print "$EOL";
}

# Caught ^Z; reset tty modes and suspend.
sub doSuspend
{
	system('stty -raw echo');
	$SIG{'TSTP'} = 'DEFAULT';
	kill 'TSTP', $$;
	goto LOOP;
}

# Caught ^C; reset tty modes and exit.
sub doKill
{
	system('stty -raw echo');
	$SIG{'INT'} = 'DEFAULT';
	kill 'INT', $$;
}

sub quit
{
	system('stty -raw echo');
	print "Bye!\n";
	exit;
}
